<?php
/**
 * @file
 *   Contains all functions relating to ec_file file transfers.
 */

/**
 * Transfer file to user if they have access.
 */
function _ec_file_download() {
  // Get requested file.
  $file = urldecode($_GET['file']);

  if (user_access('administer store')) {
    $uid = arg(2);
  }
  else {
    global $user;
    $uid = $user->uid;
  }

  // Check if the file exists
  if (!file_exists(ec_file_create_path($file))) {
    // File doesn't exist.
    drupal_not_found();
  }
  elseif (!ec_file_may_download($file, $uid)) {
    // User doesn't have access to the file.
    drupal_access_denied();
  }
  else {
    // Passed test.  Go ahead and send file.
    $headers = ec_file_build_download($file);
    if (is_array($headers)) {
      ec_file_transfer($file, $headers);
    }
  }
}

/**
 * Generate headers necessary for a file download.
 *
 * @param string $file
 *   The filename to generate headers for
 * @return
 *   Array of headers.
 */
function ec_file_build_download($file) {

  $filename = basename($file);

  // Internet Explorer for Mac cant have so big names.
  if (file_iemac_hack() && strlen($filename) > 30) {
    $filename = substr($filename, strlen($filename) - 30);
  }
  $file = ec_file_create_path($file);
  $header[] = 'Content-type: application/x-download';
  $header[] = 'Content-Disposition: attachment; filename="'. $filename .'";';
  $header[] = 'Accept-Ranges: bytes';
  $header[] = 'Content-Length: '. filesize($file);
  return $header;
}


/**
 * Transfer file using http to client. Pipes a file through Drupal to the
 * client.
 *
 * @param $source File to transfer.
 * @param $headers An array of http headers to send along with file.
 */
function ec_file_transfer($source, $headers) {
  ob_end_clean();

  foreach ($headers as $header) {
    header($header);
  }

  $source = ec_file_create_path($source);

  // Transfer file in 1024 byte chunks to save memory usage.
  $fd = fopen($source, 'rb');
  while (!feof($fd)) {
    echo fread($fd, 1024);
  }
  fclose($fd);

  // Report to site admins that the user downloaded the file
  global $user;
  watchdog('ec_file', t('The user %user has downloaded the file.',
    array('%user' => $user->uid)));

  exit();
}

/**
 * IE for Mac chokes on long filenames. This function determines
 * if the user is using IE for Mac.
 *
 * @return
 *   Boolean, true if its IE on Mac
 */
function file_iemac_hack() {
  global $HTTP_USER_AGENT;
  return !((strpos($HTTP_USER_AGENT, 'MSIE 4') === false) &&
    (strpos($HTTP_USER_AGENT, 'MSIE 5') === false) &&
    (strpos($HTTP_USER_AGENT, 'Mac') === false));
}
